@@ -1,4 +1,5 @@
-#include <cstdio>
+// #include <cstdio>
+#include <iostream>
 
 #include "HCheckConfig.h"
 #include "Highs.h"
@@ -7,13 +8,15 @@
 
 const bool dev_run = false;
 
-void runWriteReadCheckSolution(Highs& highs, const std::string model,
+void runWriteReadCheckSolution(Highs& highs, const std::string& test_name,
+                               const std::string& model,
                                const HighsModelStatus require_model_status,
                                const HighsInt write_solution_style);
 
 void runSetLpSolution(const std::string model);
 
 TEST_CASE("check-solution", "[highs_check_solution]") {
+  std::string test_name = Catch::getResultCapture().getCurrentTestName();
   std::string model = "";
   std::string model_file;
   HighsStatus read_status;
@@ -41,7 +44,8 @@ TEST_CASE("check-solution", "[highs_check_solution]") {
     REQUIRE(read_status == require_read_status);
 
     require_model_status = HighsModelStatus::kOptimal;
-    runWriteReadCheckSolution(highs, model, require_model_status,
+    test_name += "-";
+    runWriteReadCheckSolution(highs, test_name, model, require_model_status,
                               write_solution_style);
     SpecialLps special_lps;
     HighsLp lp;
@@ -50,14 +54,16 @@ TEST_CASE("check-solution", "[highs_check_solution]") {
     model = "distillation";
     special_lps.distillationMip(lp, require_model_status, optimal_objective);
     highs.passModel(lp);
-    runWriteReadCheckSolution(highs, model, require_model_status,
+    test_name += "-";
+    runWriteReadCheckSolution(highs, test_name, model, require_model_status,
                               write_solution_style);
 
     lp.clear();
     model = "primalDualInfeasible1Lp";
     special_lps.primalDualInfeasible1Lp(lp, require_model_status);
     highs.passModel(lp);
-    runWriteReadCheckSolution(highs, model, require_model_status,
+    test_name += "-";
+    runWriteReadCheckSolution(highs, test_name, model, require_model_status,
                               write_solution_style);
     // Second pass uses sparse format
     write_solution_style = kSolutionStyleSparse;
@@ -65,6 +71,7 @@ TEST_CASE("check-solution", "[highs_check_solution]") {
 }
 
 TEST_CASE("check-set-mip-solution", "[highs_check_solution]") {
+  const std::string test_name = Catch::getResultCapture().getCurrentTestName();
   HighsStatus return_status;
   const std::string model = "flugpl";
   std::string model_file =
@@ -82,10 +89,10 @@ TEST_CASE("check-set-mip-solution", "[highs_check_solution]") {
   HighsInt scratch_num_nodes = info.mip_node_count;
   if (dev_run) printf("Num nodes = %d\n", int(scratch_num_nodes));
 
-  std::string solution_file = model + ".sol";
-  if (dev_run) return_status = highs.writeSolution("");
-  return_status = highs.writeSolution(solution_file);
-  REQUIRE(return_status == HighsStatus::kOk);
+  std::string solution_file = test_name + model + ".sol";
+  if (dev_run) REQUIRE(highs.writeSolution("") == HighsStatus::kOk);
+  ;
+  REQUIRE(highs.writeSolution(solution_file) == HighsStatus::kOk);
 
   highs.clear();
 
@@ -106,7 +113,7 @@ TEST_CASE("check-set-mip-solution", "[highs_check_solution]") {
 
     highs.run();
     if (dev_run) printf("Num nodes = %d\n", int(info.mip_node_count));
-    REQUIRE(info.mip_node_count < scratch_num_nodes);
+    REQUIRE(info.mip_node_count != scratch_num_nodes);
     highs.clear();
   }
 
@@ -125,7 +132,7 @@ TEST_CASE("check-set-mip-solution", "[highs_check_solution]") {
 
     highs.run();
     if (dev_run) printf("Num nodes = %d\n", int(info.mip_node_count));
-    REQUIRE(info.mip_node_count < scratch_num_nodes);
+    REQUIRE(info.mip_node_count != scratch_num_nodes);
     highs.clear();
   }
 
@@ -155,7 +162,7 @@ TEST_CASE("check-set-mip-solution", "[highs_check_solution]") {
 
     highs.run();
     if (dev_run) printf("Num nodes = %d\n", int(info.mip_node_count));
-    REQUIRE(info.mip_node_count < scratch_num_nodes);
+    REQUIRE(info.mip_node_count != scratch_num_nodes);
     highs.clear();
   }
 
@@ -178,7 +185,7 @@ TEST_CASE("check-set-mip-solution", "[highs_check_solution]") {
 
     highs.run();
     if (dev_run) printf("Num nodes = %d\n", int(info.mip_node_count));
-    REQUIRE(info.mip_node_count < scratch_num_nodes);
+    REQUIRE(info.mip_node_count != scratch_num_nodes);
     highs.clear();
   }
 
@@ -225,7 +232,7 @@ TEST_CASE("check-set-mip-solution", "[highs_check_solution]") {
     return_status = highs.setSolution(starting_solution);
     REQUIRE(return_status == HighsStatus::kOk);
     highs.run();
-    REQUIRE(info.mip_node_count < scratch_num_nodes);
+    REQUIRE(info.mip_node_count != scratch_num_nodes);
     highs.clear();
   }
 
@@ -276,11 +283,13 @@ TEST_CASE("check-set-mip-solution", "[highs_check_solution]") {
     return_status = highs.setSolution(num_entries, index.data(), value.data());
     REQUIRE(return_status == HighsStatus::kOk);
     highs.run();
-    REQUIRE(info.mip_node_count < scratch_num_nodes);
+    REQUIRE(info.mip_node_count != scratch_num_nodes);
     highs.clear();
   }
   assert(other_tests);
   std::remove(solution_file.c_str());
+
+  highs.resetGlobalScheduler(true);
 }
 
 TEST_CASE("set-pathological-solution", "[highs_check_solution]") {
@@ -307,6 +316,8 @@ TEST_CASE("set-pathological-solution", "[highs_check_solution]") {
   highs.setSolution(solution);
   highs.run();
   REQUIRE(highs.getModelStatus() == HighsModelStatus::kUnbounded);
+
+  highs.resetGlobalScheduler(true);
 }
 
 TEST_CASE("check-set-lp-solution", "[highs_check_solution]") {
@@ -350,11 +361,14 @@ TEST_CASE("check-set-rowwise-lp-solution", "[highs_check_solution]") {
   highs.run();
   double objective2 = highs.getInfo().objective_function_value;
   REQUIRE(fabs(objective1 - objective2) / max(1.0, objective1) < 1e-5);
+
+  highs.resetGlobalScheduler(true);
 }
 
 TEST_CASE("check-set-mip-solution-extra-row", "[highs_check_solution]") {
+  const std::string test_name = Catch::getResultCapture().getCurrentTestName();
+  const std::string solution_file_name = test_name + ".sol";
   Highs highs;
-  const std::string solution_file_name = "temp.sol";
   highs.setOptionValue("output_flag", dev_run);
   highs.addVar(0, 2);
   highs.addVar(0, 2);
@@ -379,10 +393,11 @@ TEST_CASE("check-set-mip-solution-extra-row", "[highs_check_solution]") {
   highs.run();
   if (dev_run) highs.writeSolution("", 1);
   std::remove(solution_file_name.c_str());
+
+  highs.resetGlobalScheduler(true);
 }
 
 TEST_CASE("check-set-illegal-solution", "[highs_check_solution]") {
-  HighsStatus return_status;
   std::string model_file =
       std::string(HIGHS_DIR) + "/check/instances/avgas.mps";
   Highs highs;
@@ -395,7 +410,169 @@ TEST_CASE("check-set-illegal-solution", "[highs_check_solution]") {
   REQUIRE(highs.setSolution(solution) == HighsStatus::kOk);
 }
 
-void runWriteReadCheckSolution(Highs& highs, const std::string model,
+TEST_CASE("read-miplib-solution", "[highs_check_solution]") {
+  const std::string test_name = Catch::getResultCapture().getCurrentTestName();
+  HighsLp lp;
+  lp.num_col_ = 5;
+  lp.num_row_ = 1;
+  lp.sense_ = ObjSense::kMaximize;
+  lp.col_cost_ = {8, 5, 3, 11, 7};
+  lp.col_lower_.assign(lp.num_col_, 0);
+  lp.col_upper_.assign(lp.num_col_, 1);
+  lp.integrality_.assign(lp.num_col_, HighsVarType::kInteger);
+  lp.row_lower_ = {-kHighsInf};
+  lp.row_upper_ = {11};
+  lp.a_matrix_.format_ = MatrixFormat::kRowwise;
+  lp.a_matrix_.start_ = {0, 5};
+  lp.a_matrix_.index_ = {0, 1, 2, 3, 4};
+  lp.a_matrix_.value_ = {4, 3, 1, 5, 4};
+  Highs h;
+  h.setOptionValue("output_flag", dev_run);
+  h.setOptionValue("presolve", kHighsOffString);
+  REQUIRE(h.passModel(lp) == HighsStatus::kOk);
+  REQUIRE(h.run() == HighsStatus::kOk);
+  //  REQUIRE(h.writeSolution("", kSolutionStylePretty) == HighsStatus::kOk);
+  const std::vector<double>& col_value = h.getSolution().col_value;
+  std::string miplib_sol_file = test_name + ".sol";
+  FILE* file = fopen(miplib_sol_file.c_str(), "w");
+  REQUIRE(file != 0);
+  fprintf(file, "=obj= 22\n");
+  for (HighsInt iCol = 0; iCol < lp.num_col_; iCol++) {
+    std::string col_name = "c" + std::to_string(int(iCol));
+    lp.col_names_.push_back(col_name);
+    if (std::fabs(col_value[iCol]) < 1e-2) continue;
+    std::string line = col_name + " 1\n";
+    fprintf(file, "%s", line.c_str());
+  }
+  fclose(file);
+  // Can't read file yet, as model has no column names
+  REQUIRE(h.readSolution(miplib_sol_file) == HighsStatus::kError);
+
+  // Pass model again now that column names have been defined
+
+  REQUIRE(h.passModel(lp) == HighsStatus::kOk);
+  //  REQUIRE(h.writeModel("miplib.mps") == HighsStatus::kOk);
+  REQUIRE(h.readSolution(miplib_sol_file) == HighsStatus::kOk);
+  REQUIRE(h.run() == HighsStatus::kOk);
+  std::remove(miplib_sol_file.c_str());
+
+  h.resetGlobalScheduler(true);
+}
+
+TEST_CASE("read-lp-file-solution", "[highs_check_solution]") {
+  const std::string test_name = Catch::getResultCapture().getCurrentTestName();
+  const std::string model_file_name = test_name + ".lp";
+  const std::string solution_file_name = test_name + ".sol";
+  const bool with_names = false;
+  HighsLp lp;
+  lp.num_col_ = 3;
+  lp.num_row_ = 2;
+  lp.col_cost_ = {0, 1, 1};
+  lp.col_lower_ = {0, 10, 0};
+  lp.col_upper_ = {kHighsInf, kHighsInf, kHighsInf};
+  if (with_names) lp.col_names_ = {"x", "y", "z"};
+  lp.row_lower_ = {1, -kHighsInf};
+  lp.row_upper_ = {kHighsInf, 2};
+  if (with_names) lp.row_names_ = {"r-lo", "r-up"};
+  lp.a_matrix_.start_ = {0, 2, 2, 4};
+  lp.a_matrix_.index_ = {0, 1, 0, 1};
+  lp.a_matrix_.value_ = {1, 1, 1, 1};
+  lp.integrality_ = {HighsVarType::kContinuous, HighsVarType::kContinuous,
+                     HighsVarType::kInteger};
+  Highs h;
+  h.setOptionValue("output_flag", dev_run);
+  REQUIRE(h.passModel(lp) == HighsStatus::kOk);
+  h.run();
+  h.writeModel(model_file_name);
+  h.writeSolution(solution_file_name);
+
+  h.readModel(model_file_name);
+  h.writeModel("");
+  h.readSolution(solution_file_name);
+  h.run();
+
+  std::remove(model_file_name.c_str());
+  std::remove(solution_file_name.c_str());
+
+  h.resetGlobalScheduler(true);
+}
+
+TEST_CASE("read-lp-file-basis", "[highs_check_solution]") {
+  const std::string test_name = Catch::getResultCapture().getCurrentTestName();
+  const std::string model_file_name = test_name + ".lp";
+  const std::string basis_file_name = test_name + ".bas";
+  const bool with_names = false;
+  HighsLp lp;
+  lp.num_col_ = 3;
+  lp.num_row_ = 2;
+  lp.col_cost_ = {0, 1, 1};
+  lp.col_lower_ = {0, 10, 0};
+  lp.col_upper_ = {kHighsInf, kHighsInf, kHighsInf};
+  if (with_names) lp.col_names_ = {"x", "y", "z"};
+  lp.row_lower_ = {1, -kHighsInf};
+  lp.row_upper_ = {kHighsInf, 2};
+  if (with_names) lp.row_names_ = {"r-lo", "r-up"};
+  lp.a_matrix_.start_ = {0, 2, 2, 4};
+  lp.a_matrix_.index_ = {0, 1, 0, 1};
+  lp.a_matrix_.value_ = {1, 1, 1, 1};
+  Highs h;
+  h.setOptionValue("output_flag", dev_run);
+  REQUIRE(h.passModel(lp) == HighsStatus::kOk);
+  h.run();
+  // Optimally x - basic; y - lower; z - lower
+  h.writeModel(model_file_name);
+  h.writeSolution("", 1);
+  h.writeBasis("");
+  h.writeBasis(basis_file_name);
+
+  h.readModel(model_file_name);
+  // Variables now ordered y; z; x
+  h.writeModel("");
+  h.readBasis(basis_file_name);
+  // Old read basis yields initial basis: y - basic; z - lower; x -
+  // lower, using basis for original ordering with new ordering. Not
+  // optimal - in fact basis matrix B = [0] is singular!
+  h.run();
+  REQUIRE(h.getInfo().simplex_iteration_count == 0);
+
+  std::remove(model_file_name.c_str());
+  std::remove(basis_file_name.c_str());
+
+  h.resetGlobalScheduler(true);
+}
+
+TEST_CASE("read-lp-file-rgn", "[highs_check_solution]") {
+  const std::string test_name = Catch::getResultCapture().getCurrentTestName();
+  const std::string filename =
+      std::string(HIGHS_DIR) + "/check/instances/rgn.mps";
+  const std::string model_file_name = test_name + ".lp";
+  const std::string solution_file_name = test_name + ".sol";
+  Highs h;
+  h.setOptionValue("output_flag", dev_run);
+  REQUIRE(h.readModel(filename) == HighsStatus::kOk);
+  REQUIRE(h.run() == HighsStatus::kOk);
+  REQUIRE(h.writeSolution(solution_file_name) == HighsStatus::kOk);
+  REQUIRE(h.writeModel(model_file_name) == HighsStatus::kOk);
+
+  REQUIRE(h.readModel(model_file_name) == HighsStatus::kOk);
+  REQUIRE(h.readSolution(solution_file_name) == HighsStatus::kOk);
+  bool valid;
+  bool integral;
+  bool feasible;
+  REQUIRE(h.assessPrimalSolution(valid, integral, feasible) ==
+          HighsStatus::kOk);
+  REQUIRE(valid);
+  REQUIRE(integral);
+  REQUIRE(feasible);
+
+  std::remove(model_file_name.c_str());
+  std::remove(solution_file_name.c_str());
+
+  h.resetGlobalScheduler(true);
+}
+
+void runWriteReadCheckSolution(Highs& highs, const std::string& test_name,
+                               const std::string& model,
                                const HighsModelStatus require_model_status,
                                const HighsInt write_solution_style) {
   HighsStatus run_status;
@@ -409,13 +586,19 @@ void runWriteReadCheckSolution(Highs& highs, const std::string model,
   status = highs.getModelStatus();
   REQUIRE(status == require_model_status);
 
-  solution_file = model + ".sol";
+  solution_file = test_name + model + ".sol";
   if (dev_run)
     printf("Writing solution in style %d to %s\n", int(write_solution_style),
            solution_file.c_str());
-  if (dev_run) return_status = highs.writeSolution("", write_solution_style);
-  return_status = highs.writeSolution(solution_file, write_solution_style);
-  REQUIRE(return_status == HighsStatus::kOk);
+  // For models without names, Highs::writeSolution will return
+  // HighsStatus::kWarning
+  HighsStatus require_status = highs.getLp().col_names_.size()
+                                   ? HighsStatus::kOk
+                                   : HighsStatus::kWarning;
+  REQUIRE(highs.writeSolution(solution_file, write_solution_style) ==
+          require_status);
+  if (dev_run)
+    REQUIRE(highs.writeSolution("", write_solution_style) == HighsStatus::kOk);
 
   const bool& value_valid = highs.getSolution().value_valid;
   bool valid, integral, feasible;
@@ -444,6 +627,8 @@ void runWriteReadCheckSolution(Highs& highs, const std::string model,
   REQUIRE(status == require_model_status);
 
   std::remove(solution_file.c_str());
+
+  highs.resetGlobalScheduler(true);
 }
 
 void runSetLpSolution(const std::string model) {
@@ -511,4 +696,45 @@ void runSetLpSolution(const std::string model) {
   highs.clear();
 
   std::remove(solution_file.c_str());
+
+  highs.resetGlobalScheduler(true);
+}
+
+TEST_CASE("miplib-sol-file", "[highs_filereader]") {
+  const std::string test_name = Catch::getResultCapture().getCurrentTestName();
+  std::string sol_file = test_name + ".sol";
+  std::string lp_file = test_name + ".lp";
+  FILE* file = fopen(lp_file.c_str(), "w");
+  std::string file_content =
+      "Minimize\n obj: 2 sel_2 + sel_3\nSubject To\nr0: sel_0 - sel_1 + sel_4 "
+      ">= "
+      "2\nEnd\n";
+  if (dev_run) printf("Using .lp file\n%s", file_content.c_str());
+  fprintf(file, "%s", file_content.c_str());
+  fclose(file);
+  Highs h;
+  h.setOptionValue("output_flag", dev_run);
+  REQUIRE(h.readModel(lp_file) == HighsStatus::kOk);
+
+  file = fopen(sol_file.c_str(), "w");
+  file_content =
+      "=obj= 203672547.1\nsel_0 1\nsel_1 0\nsel_2 0\nsel_3 0\nsel_4 1\n";
+  if (dev_run) printf("Using .sol file\n%s", file_content.c_str());
+  fprintf(file, "%s", file_content.c_str());
+  fclose(file);
+  REQUIRE(h.readSolution(sol_file) == HighsStatus::kOk);
+
+  std::vector<double> solution = h.getSolution().col_value;
+  REQUIRE(solution[0] == 0);
+  REQUIRE(solution[1] == 0);
+  REQUIRE(solution[2] == 1);
+  REQUIRE(solution[3] == 0);
+  REQUIRE(solution[4] == 1);
+
+  REQUIRE(h.run() == HighsStatus::kOk);
+
+  std::remove(lp_file.c_str());
+  std::remove(sol_file.c_str());
+
+  h.resetGlobalScheduler(true);
 }
