@@ -81,6 +81,8 @@ TEST_CASE("postsolve-no-basis", "[highs_test_presolve]") {
     solution.col_dual.clear();
     solution.row_dual.clear();
   }
+
+  highs.resetGlobalScheduler(true);
 }
 
 TEST_CASE("presolve-solve-postsolve-mip", "[highs_test_presolve]") {
@@ -170,6 +172,8 @@ TEST_CASE("empty-row", "[highs_test_presolve]") {
   const HighsBasis& basis = highs.getBasis();
   REQUIRE(HighsInt(solution.row_value.size()) == lp.num_row_);
   REQUIRE(HighsInt(basis.row_status.size()) == lp.num_row_);
+
+  highs.resetGlobalScheduler(true);
 }
 
 void presolveSolvePostsolve(const std::string& model_file,
@@ -180,7 +184,7 @@ void presolveSolvePostsolve(const std::string& model_file,
   highs1.setOptionValue("output_flag", dev_run);
   HighsStatus return_status;
   highs0.readModel(model_file);
-  if (solve_relaxation) highs0.setOptionValue("solver", kSimplexString);
+  highs0.setOptionValue("solve_relaxation", solve_relaxation);
   return_status = highs0.presolve();
   REQUIRE(return_status == HighsStatus::kOk);
   HighsPresolveStatus model_presolve_status = highs0.getModelPresolveStatus();
@@ -190,7 +194,7 @@ void presolveSolvePostsolve(const std::string& model_file,
   }
   HighsLp lp = highs0.getPresolvedLp();
   highs1.passModel(lp);
-  if (solve_relaxation) highs1.setOptionValue("solver", kSimplexString);
+  highs1.setOptionValue("solve_relaxation", solve_relaxation);
   highs1.setOptionValue("presolve", kHighsOffString);
   highs1.run();
   HighsSolution solution = highs1.getSolution();
@@ -202,7 +206,7 @@ void presolveSolvePostsolve(const std::string& model_file,
     REQUIRE(model_status == HighsModelStatus::kUnknown);
     const double dl_objective_value =
         std::fabs(highs0.getInfo().objective_function_value - objective_value);
-    REQUIRE(dl_objective_value < 1e-12);
+    REQUIRE(dl_objective_value < 1e-9);
     REQUIRE(highs0.getInfo().primal_solution_status == kSolutionStatusFeasible);
     double mip_feasibility_tolerance;
     highs0.getOptionValue("mip_feasibility_tolerance",
@@ -211,12 +215,15 @@ void presolveSolvePostsolve(const std::string& model_file,
             mip_feasibility_tolerance);
   } else {
     HighsBasis basis = highs1.getBasis();
+    REQUIRE(basis.valid);
     return_status = highs0.postsolve(solution, basis);
     REQUIRE(return_status == HighsStatus::kOk);
     HighsModelStatus model_status = highs0.getModelStatus();
     REQUIRE(model_status == HighsModelStatus::kOptimal);
     REQUIRE(highs0.getInfo().simplex_iteration_count <= 0);
   }
+
+  highs1.resetGlobalScheduler(true);
 }
 
 HighsStatus zeroCostColSing() {
@@ -252,6 +259,9 @@ HighsStatus zeroCostColSing() {
   assert(status == HighsStatus::kOk);
 
   status = highs.run();
+
+  highs.resetGlobalScheduler(true);
+
   return status;
 }
 
@@ -309,6 +319,9 @@ HighsStatus colSingDoubletonEquality() {
   assert(status == HighsStatus::kOk);
 
   status = highs.run();
+
+  highs.resetGlobalScheduler(true);
+
   return status;
 }
 
@@ -365,6 +378,9 @@ HighsStatus colSingDoubletonInequality() {
   assert(status == HighsStatus::kOk);
 
   status = highs.run();
+
+  highs.resetGlobalScheduler(true);
+
   return status;
 }
 
@@ -402,6 +418,9 @@ HighsStatus twoColSingDoubletonEquality() {
   assert(status == HighsStatus::kOk);
 
   status = highs.run();
+
+  highs.resetGlobalScheduler(true);
+
   return status;
 }
 
@@ -440,6 +459,9 @@ HighsStatus twoColSingDoubletonInequality() {
 
   highs.run();
   status = highs.run();
+
+  highs.resetGlobalScheduler(true);
+
   return status;
 }
 
@@ -527,6 +549,9 @@ HighsStatus issue425() {
   assert(status == HighsStatus::kOk);
 
   status = highs.run();
+
+  highs.resetGlobalScheduler(true);
+
   return status;
 }
 
@@ -577,7 +602,8 @@ TEST_CASE("postsolve-reduced-to-empty", "[highs_test_presolve]") {
 }
 
 TEST_CASE("write-presolved-model", "[highs_test_presolve]") {
-  std::string presolved_model_file = "temp.mps";
+  const std::string test_name = Catch::getResultCapture().getCurrentTestName();
+  const std::string presolved_model_file = test_name + ".mps";
   std::string model_file =
       std::string(HIGHS_DIR) + "/check/instances/afiro.mps";
   Highs highs;
@@ -600,4 +626,240 @@ TEST_CASE("write-presolved-model", "[highs_test_presolve]") {
   // simplex_iteration_count is -1
   REQUIRE(highs.getInfo().simplex_iteration_count == -1);
   std::remove(presolved_model_file.c_str());
+
+  highs1.resetGlobalScheduler(true);
+}
+
+TEST_CASE("presolve-slacks", "[highs_test_presolve]") {
+  // This LP reduces to empty, because the equation is a doubleton
+  HighsLp lp;
+  lp.num_col_ = 2;
+  lp.num_row_ = 1;
+  lp.col_cost_ = {1, 0};
+  lp.col_lower_ = {0, 0};
+  lp.col_upper_ = {kHighsInf, kHighsInf};
+  lp.row_lower_ = {1};
+  lp.row_upper_ = {1};
+  lp.a_matrix_.start_ = {0, 1, 2};
+  lp.a_matrix_.index_ = {0, 0};
+  lp.a_matrix_.value_ = {1, 1};
+  Highs h;
+  h.setOptionValue("output_flag", dev_run);
+  REQUIRE(h.passModel(lp) == HighsStatus::kOk);
+  REQUIRE(h.presolve() == HighsStatus::kOk);
+  REQUIRE(h.getPresolvedLp().num_col_ == 0);
+  REQUIRE(h.getPresolvedLp().num_row_ == 0);
+
+  lp.num_col_ = 4;
+  lp.num_row_ = 2;
+  lp.col_cost_ = {-10, -25, 0, 0};
+  lp.col_lower_ = {0, 0, 0, 0};
+  lp.col_upper_ = {kHighsInf, kHighsInf, kHighsInf, kHighsInf};
+  lp.row_lower_ = {80, 120};
+  lp.row_upper_ = {80, 120};
+  lp.a_matrix_.start_ = {0, 2, 4, 5, 6};
+  lp.a_matrix_.index_ = {0, 1, 0, 1, 0, 1};
+  lp.a_matrix_.value_ = {1, 1, 2, 4, 1, 1};
+  REQUIRE(h.setOptionValue("presolve_remove_slacks", true) == HighsStatus::kOk);
+  REQUIRE(h.passModel(lp) == HighsStatus::kOk);
+  REQUIRE(h.run() == HighsStatus::kOk);
+  REQUIRE(h.presolve() == HighsStatus::kOk);
+  REQUIRE(h.getPresolvedLp().num_col_ == 2);
+  REQUIRE(h.getPresolvedLp().num_row_ == 2);
+
+  h.resetGlobalScheduler(true);
+}
+
+TEST_CASE("presolve-issue-2095", "[highs_test_presolve]") {
+  std::string model_file =
+      std::string(HIGHS_DIR) + "/check/instances/issue-2095.mps";
+  Highs highs;
+  highs.setOptionValue("output_flag", dev_run);
+  highs.readModel(model_file);
+  REQUIRE(highs.presolve() == HighsStatus::kOk);
+  REQUIRE(highs.getModelPresolveStatus() == HighsPresolveStatus::kReduced);
+}
+
+TEST_CASE("presolve-only-at-root", "[highs_test_presolve]") {
+  std::string model_file = std::string(HIGHS_DIR) + "/check/instances/rgn.mps";
+
+  Highs highs;
+  highs.setOptionValue("output_flag", dev_run);
+  // Allow only presolve at root node
+  highs.setOptionValue("mip_root_presolve_only", true);
+  highs.readModel(model_file);
+  REQUIRE(highs.run() == HighsStatus::kOk);
+
+  highs.resetGlobalScheduler(true);
+}
+
+TEST_CASE("lifting-for-probing", "[highs_test_presolve]") {
+  std::string model_file =
+      std::string(HIGHS_DIR) + "/check/instances/gesa2.mps";
+
+  Highs highs;
+  highs.setOptionValue("output_flag", dev_run);
+  // Enable lifting for probing
+  highs.setOptionValue("mip_lifting_for_probing", 1);
+  highs.readModel(model_file);
+  REQUIRE(highs.presolve() == HighsStatus::kOk);
+}
+
+TEST_CASE("presolve-issue-2388", "[highs_test_presolve]") {
+  std::string model_file =
+      std::string(HIGHS_DIR) + "/check/instances/issue-2388.lp";
+  Highs highs;
+  highs.setOptionValue("output_flag", dev_run);
+  highs.readModel(model_file);
+  REQUIRE(highs.presolve() == HighsStatus::kOk);
+  REQUIRE(highs.getModelPresolveStatus() ==
+          HighsPresolveStatus::kReducedToEmpty);
+}
+
+TEST_CASE("presolve-issue-2402", "[highs_test_presolve]") {
+  std::string model_file =
+      std::string(HIGHS_DIR) + "/check/instances/issue-2402.mps";
+  Highs highs;
+  highs.setOptionValue("output_flag", dev_run);
+  highs.readModel(model_file);
+  REQUIRE(highs.presolve() == HighsStatus::kOk);
+  REQUIRE(highs.getModelPresolveStatus() == HighsPresolveStatus::kInfeasible);
+}
+
+TEST_CASE("presolve-issue-2446", "[highs_test_presolve]") {
+  std::string model_file =
+      std::string(HIGHS_DIR) + "/check/instances/issue-2446.mps";
+  Highs highs;
+  highs.setOptionValue("output_flag", dev_run);
+  highs.readModel(model_file);
+  REQUIRE(highs.presolve() == HighsStatus::kOk);
+  REQUIRE(highs.getModelPresolveStatus() == HighsPresolveStatus::kReduced);
+}
+
+TEST_CASE("presolve-solve-postsolve-no-col-dual", "[highs_test_presolve]") {
+  Highs highs;
+  highs.setOptionValue("output_flag", dev_run);
+  std::string model_file =
+      std::string(HIGHS_DIR) + "/check/instances/afiro.mps";
+  highs.readModel(model_file);
+  highs.presolve();
+  HighsLp presolved_lp = highs.getPresolvedLp();
+  Highs highs1;
+  highs1.setOptionValue("output_flag", dev_run);
+  highs1.setOptionValue("presolve", kHighsOffString);
+  highs1.passModel(presolved_lp);
+  highs1.run();
+  HighsSolution solution = highs1.getSolution();
+
+  // Perform postsolve using the optimal solution and basis for the
+  // presolved model
+  REQUIRE(highs.postsolve(solution) == HighsStatus::kOk);
+
+  // If row duals are supplied, then column duals must also be suppplied
+  solution.col_dual.clear();
+  REQUIRE(highs.postsolve(solution) == HighsStatus::kError);
+
+  highs.resetGlobalScheduler(true);
+}
+
+TEST_CASE("presolve-egout-ac", "[highs_test_presolve]") {
+  // Tests the case where, for this model when run_crossover is off,
+  // sparsify is used to reduce the LP to empty. However, when
+  // starting from an empty LP, basis postsolve is always used
+  std::string model_file =
+      std::string(HIGHS_DIR) + "/check/instances/egout-ac.mps";
+  Highs h;
+  h.setOptionValue("output_flag", dev_run);
+  REQUIRE(h.setOptionValue("presolve_rule_logging", true) == HighsStatus::kOk);
+  if (dev_run)
+    REQUIRE(h.setOptionValue("log_dev_level", 1) == HighsStatus::kOk);
+  REQUIRE(h.readModel(model_file) == HighsStatus::kOk);
+  // Firstly check that pure presolve reduces the LP to empty
+  REQUIRE(h.presolve() == HighsStatus::kOk);
+  // Ensure that sparsify isn't called
+  REQUIRE(h.getPresolveLog().rule[kPresolveRuleSparsify].call == 0);
+
+  const HighsLp& presolved_lp = h.getPresolvedLp();
+  REQUIRE(presolved_lp.num_col_ == 0);
+  REQUIRE(presolved_lp.num_row_ == 0);
+
+  HighsSolution solution;
+  HighsBasis basis;
+  basis.useful = true;
+  solution.value_valid = true;
+  solution.dual_valid = true;
+  // Check that postsolve with the empty solution and basis runs OK -
+  // ie doesn't trigger assert due to sparsify having been used
+  REQUIRE(h.postsolve(solution, basis) == HighsStatus::kOk);
+
+  // Check that using IPM with crossover runs OK without using
+  // sparsify
+  REQUIRE(h.setOptionValue("solver", kIpmString) == HighsStatus::kOk);
+  REQUIRE(h.run() == HighsStatus::kOk);
+  REQUIRE(h.getPresolveLog().rule[kPresolveRuleSparsify].call == 0);
+
+  // Check that pure presolve reduces the LP to empty without using
+  // sparsify
+  REQUIRE(h.presolve() == HighsStatus::kOk);
+  REQUIRE(h.getPresolveLog().rule[kPresolveRuleSparsify].call == 0);
+  REQUIRE(h.postsolve(solution, basis) == HighsStatus::kOk);
+
+  // Check that pure presolve reduces the LP to empty using sparsify
+  // when lp_presolve_requires_basis_postsolve is false
+  bool lp_presolve_requires_basis_postsolve = false;
+  REQUIRE(h.setOptionValue("lp_presolve_requires_basis_postsolve",
+                           lp_presolve_requires_basis_postsolve) ==
+          HighsStatus::kOk);
+  REQUIRE(h.presolve() == HighsStatus::kOk);
+  REQUIRE(h.getPresolveLog().rule[kPresolveRuleSparsify].call > 0);
+  REQUIRE(h.postsolve(solution, basis) == HighsStatus::kOk);
+  REQUIRE(h.getOptions().lp_presolve_requires_basis_postsolve ==
+          lp_presolve_requires_basis_postsolve);
+
+  // Now, with crossover off
+  REQUIRE(h.setOptionValue("run_crossover", kHighsOffString) ==
+          HighsStatus::kOk);
+
+  // Now reset lp_presolve_requires_basis_postsolve default to true,
+  // to test whether it's set false due to running IPM without
+  // crossover
+  lp_presolve_requires_basis_postsolve = true;
+  REQUIRE(h.setOptionValue("lp_presolve_requires_basis_postsolve",
+                           lp_presolve_requires_basis_postsolve) ==
+          HighsStatus::kOk);
+
+  REQUIRE(h.clearSolver() == HighsStatus::kOk);
+  REQUIRE(h.run() == HighsStatus::kOk);
+  REQUIRE(h.getPresolveLog().rule[kPresolveRuleSparsify].call > 0);
+  // Ensure that lp_presolve_requires_basis_postsolve has been reset
+  // to true, after being set false before presolve when using IPM
+  // without crossover
+  REQUIRE(h.getOptions().lp_presolve_requires_basis_postsolve ==
+          lp_presolve_requires_basis_postsolve);
+
+  h.resetGlobalScheduler(true);
+}
+
+TEST_CASE("dual-bound-tightening", "[highs_test_presolve]") {
+  std::string model_file =
+      std::string(HIGHS_DIR) + "/check/instances/gesa2.mps";
+
+  Highs highs;
+  highs.setOptionValue("output_flag", dev_run);
+  highs.readModel(model_file);
+
+  // complement variables to get code coverage
+  HighsLp lp = highs.getLp();
+  std::transform(lp.a_matrix_.value_.begin(), lp.a_matrix_.value_.end(),
+                 lp.a_matrix_.value_.begin(), [](double v) { return -v; });
+  std::transform(lp.col_cost_.begin(), lp.col_cost_.end(), lp.col_cost_.begin(),
+                 [](double v) { return -v; });
+  std::transform(lp.col_upper_.begin(), lp.col_upper_.end(),
+                 lp.col_upper_.begin(), [](double v) { return -v; });
+  std::transform(lp.col_lower_.begin(), lp.col_lower_.end(),
+                 lp.col_lower_.begin(), [](double v) { return -v; });
+  std::swap(lp.col_lower_, lp.col_upper_);
+
+  highs.passModel(lp);
+  REQUIRE(highs.presolve() == HighsStatus::kOk);
 }
